from datetime import datetime, timedelta
from typing import Annotated

from pymongo import MongoClient
from fastapi import Depends, HTTPException, status
from fastapi.security import OAuth2PasswordBearer
from jose import JWTError, jwt
from passlib.context import CryptContext

from ..models import users, token
from ..dependencies import database, cookie

from authlib.integrations.starlette_client import OAuth
import httpx, os 

SECRET_KEY = "09d25e094faa6ca2556c818166b7a9563b93f7099f6f0f4caa6cf63b88e8d3e7"
ALGORITHM = "HS256"
ACCESS_TOKEN_EXPIRE_MINUTES = 30

pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")

oauth = OAuth()
oauth.register(
    name="google",
    client_id=os.environ["GOOGLE_CLIENT_ID"],
    client_secret=os.environ["GOOGLE_CLIENT_SECRET"],
    authorize_url="https://accounts.google.com/o/oauth2/auth",
    access_token_url="https://oauth2.googleapis.com/token",
    client_kwargs={"scope": "openid email profile"},
)

oauth.register(
    name="facebook",
    client_id=os.environ["FACEBOOK_CLIENT_ID"],
    client_secret=os.environ["FACEBOOK_CLIENT_SECRET"],
    authorize_url="https://www.facebook.com/v12.0/dialog/oauth",
    access_token_url="https://graph.facebook.com/v12.0/oauth/access_token",
    client_kwargs={"scope": "email public_profile"},
)

oauth2_scheme = cookie.OAuth2PasswordBearerWithCookie(tokenUrl="token")

def create_access_token(data: dict, expires_delta: timedelta | None = None):
    to_encode = data.copy()
    expire = datetime.utcnow() + (expires_delta or timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES))
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)
    return encoded_jwt

async def authenticate_oauth(provider: str, token: str):
    """Validate OAuth token and get user info."""
    if provider == "google":
        url = f"https://www.googleapis.com/oauth2/v3/userinfo?access_token={token}"
    elif provider == "facebook":
        url = f"https://graph.facebook.com/me?fields=id,name,email,picture&access_token={token}"
    else:
        raise HTTPException(status_code=400, detail="Unsupported provider")

    async with httpx.AsyncClient() as client:
        response = await client.get(url)
        if response.status_code != 200:
            raise HTTPException(status_code=400, detail="Invalid OAuth token")

    user_info = response.json()
    email = user_info.get("email")
    if not email:
        raise HTTPException(status_code=400, detail="Email not provided by provider")

    user_repository = users.UserRepository(database=database.database)
    user = user_repository.find_one_by({'email': email})

    if not user:
        user = users.User(
            username=email,
            email=email,
            profile_picture=user_info.get("picture", {}).get("data", {}).get("url", ""),
            status=1,
        )
        user_repository.save(user)

    return user

def verify_password(plain_password, hashed_password):
    return pwd_context.verify(plain_password, hashed_password)

def get_password_hash(password):
    return pwd_context.hash(password)

def get_user(username: str):
    user_repository = users.UserRepository(database=database.database)
    user = user_repository.find_one_by({'username': username})
    return user

def authenticate_user(username: str, password: str):
    
    user = get_user(username)
    if not user:
        return False
    if not verify_password(password, user.password):
        return False
    user.connected_at = datetime.today()
    user_repository = users.UserRepository(database=database.database)
    user_repository.save(user)
    return user

def create_access_token(data: dict, expires_delta: timedelta | None = None):
    to_encode = data.copy()
    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=15)
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)
    return encoded_jwt

async def get_current_user(token_str: Annotated[str, Depends(oauth2_scheme)]):
    credentials_exception = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Could not validate credentials",
        headers={"WWW-Authenticate": "Bearer"},
    )
    try:
        payload = jwt.decode(token_str, SECRET_KEY, algorithms=[ALGORITHM])
        username: str = payload.get("sub")
        if username is None:
            raise credentials_exception
        token_data = token.TokenData(username=username)
    except JWTError:
        raise credentials_exception
    
    user = get_user(token_data.username)
    if user is None:
        raise credentials_exception
    return user

async def get_current_active_user(
    current_user: Annotated[users.User, Depends(get_current_user)]
):
    if current_user.status == 0:
        raise HTTPException(status_code=400, detail="Inactive user")
    return current_user